#if !defined(FPP_H)
#define FPP_H

#include <config.h>

#if defined(HAVE_CTYPE_H)
#include <ctype.h>
#endif

#if defined(HAVE_STDIO_H)
#include <stdio.h>
#endif

#if defined(HAVE_STRING_H)
#include <string.h>
#endif

#include <xstdlib.h>

#ifndef EXTERN
#define EXTERN extern
#endif

#if STDC
#define ERR_PREFIX		"\?\? "	/* not a trigraph! */
#else
#define ERR_PREFIX		"?? "
#endif

#if defined(FFLUSH)
#undef FFLUSH
#endif
#define FFLUSH			(void)fflush

#if defined(FPRINTF)
#undef FPRINTF
#endif
#define FPRINTF			(void)fprintf

#define MAXLEVEL		25	/* maximum conditional nesting */
#define MAXLINE			1023	/* maximum input line length */
#define MSG_PREFIX		"!! "
#define NOOP
#define OUT_FIXED_PREFIX	"C-fpp-"
#define OUT_FREE_PREFIX		"!-fpp-"
#define PERMANENT(s)		((s[0] == '_') && (s[1] == '_'))

#if defined(PRINTF)
#undef PRINTF
#endif
#define PRINTF			(void)printf

#if defined(SPRINTF)
#undef SPRINTF
#endif
#define SPRINTF			(void)sprintf

#define STDIN			"--"

#define elementsof(array)	(sizeof(array)/sizeof(array[0]))

#define initial(c)		(fpp_line[0] == (c))
#define is_comment()		(((src_form == SF_FIXED) && is_fixed_comment()) || \
				 ((src_form == SF_FREE) && is_free_comment()))
#define is_fixed_comment()	( initial('C') || initial('c') || initial('*') )
#define is_free_comment()	initial('!')
#define is_id_line()		(is_comment() && (strncmp(&fpp_line[1],"-fpp-",5) == 0))
#define is_token_start(c)	(isalpha(c) || ((c) == '_'))
#define is_token_letter(c)	(isalnum(c) || ((c) == '_'))
#define print_line(p)		print_and_check(p)
#define skip_whitespace(p)	while(isspace(*p)) ++p
#define stddbg			stderr

#define SF_UNKNOWN	0		/* values of g_src_form and src_form */
#define SF_FIXED	1
#define SF_FREE		2

EXTERN int branch_done[MAXLEVEL];	/* for conditional branch records */
EXTERN char err_line[MAXLINE+1];	/* error message buffer */
EXTERN char expression[2*MAXLINE+1];	/* for expression expansion */
EXTERN char *ee[MAXLINE+1];		/* pointers into expression[] */
EXTERN int FNR;				/* record number in current file */
EXTERN int fpp_code;			/* exit code */
EXTERN int fpp_debug;			/* non-zero if debugging */
EXTERN FILE *fpp_infile;		/* pointer to current input file */
EXTERN char fpp_line[MAXLINE+1];	/* current input line */
EXTERN const char *fpp_name;		/* current input file name */
EXTERN char fpp_number[18];		/* current input line number */
EXTERN int g_src_form;			/* one of SF_xxx values */
EXTERN int level;			/* conditional nesting level */
EXTERN int ke;				/* current expression fields */
EXTERN int lineno[MAXLEVEL];		/* for conditional branch records */
EXTERN int ne;				/* number of expression fields */
EXTERN int NR;				/* record number in input */
EXTERN const char *out_prefix;		/* either OUT_FIXED_PREFIX or OUT_FREE_PREFIX */
EXTERN int RSTART;			/* regexp match begin in fpp_line */
EXTERN int RLENGTH;			/* regexp match length in fpp_line */
EXTERN int selected[MAXLEVEL];		/* for conditional branch flagging */
EXTERN int src_form;			/* one of SF_xxx values */
EXTERN size_t src_line_length;		/* 72 or 132 */
EXTERN char *TOKEN;			/* pointer to pp-token in fpp_line */

const char	*concat ARGS((const char *s_, const char *t_));
const char	*defined ARGS((void));
int		doarg ARGS((const char *s_));
void		dodef ARGS((const char *s_));
void		dodef2 ARGS((const char *thename_, const char *thevalue_,
			     int permanent_));
void		dofile ARGS((const char *filename_));
void		doundef ARGS((const char *s_));
char		*dtos ARGS((double d_));
void		error ARGS((const char *msg1_, const char *msg2_,
			    const char *msg3_, int dump_lineno_));
const char	*eval ARGS((const char *s_));
char		*expand ARGS((const char *s_));
const char	*expr ARGS((void));
const char	*factor ARGS((void));
const char	*fpp_dbgprefix ARGS((void));
const char	*getdef ARGS((const char *s_));
#define getline fpp_getline		/* avoid conflict with GNU getline exposed by _GNU_SOURCE */
int		getline ARGS((void));
const char	*getname ARGS((const char *s_, int start_));
const char	*identification ARGS((void));
void		init ARGS((void));
int		install ARGS((const char *thename_, const char *thevalue_));
int 		is_macro_line ARGS((void));
int 		is_pp_line ARGS((void));
int		lookup ARGS((const char *thename_));
void		macro_line ARGS((void));
int		main ARGS((int argc_, char *argv_[]));
int		match_expr ARGS((const char *s_));
int		name ARGS((const char *s_));
void		normal_line ARGS((void));
int		number ARGS((const char *s_));
const char	*power ARGS((void));
void		pp_line ARGS((void));
void		print_and_check ARGS((const char* line));
void		process ARGS((void));
void		quit ARGS((const char *message_, int dump_lineno_));
void		set_src_form ARGS((const char *filename));
char		*Strdup ARGS((const char *s_));
int		stricmp ARGS((const char *s_, const char *t_));
int		strnicmp ARGS((const char *s_, const char *t_, size_t n_));
const char	*term ARGS((void));
void		tokenize ARGS((const char *s_));
char		*trim ARGS((char *s_));
const char	*version ARGS((void));
void		warning ARGS((const char *msg1, const char *msg2,
			      const char *msg3));
#endif /* !defined(FPP_H) */
